@@ -9,6 +9,7 @@ from http.server import HTTPServer as BaseHTTPServer, SimpleHTTPRequestHandler
 import pkg_resources
 from distutils import dir_util
 from gradio import inputs, outputs
+import time
 import json
 from gradio.tunneling import create_tunnel
 import urllib.request
@@ -16,6 +17,7 @@ from shutil import copyfile
 import requests
 import sys
 import analytics
+import csv
 
 
 INITIAL_PORT_VALUE = int(os.getenv(
@@ -36,8 +38,6 @@ CONFIG_FILE = "static/config.json"
 ASSOCIATION_PATH_IN_STATIC = "static/apple-app-site-association"
 ASSOCIATION_PATH_IN_ROOT = "apple-app-site-association"
 
-FLAGGING_DIRECTORY = 'static/flagged/'
-FLAGGING_FILENAME = 'data.txt'
 analytics.write_key = "uxIFddIEuuUcFLf9VgH2teTEtPlWdkNy"
 analytics_url = 'https://api.gradio.app/'
 
@@ -172,20 +172,6 @@ def serve_files_in_background(interface, port, directory_to_serve=None, server_n
                 prediction, durations = interface.process(raw_input)
 
                 output = {"data": prediction, "durations": durations}
-                if interface.saliency is not None:
-                    saliency = interface.saliency(raw_input, prediction)
-                    output['saliency'] = saliency.tolist()
-                # if interface.always_flag:
-                #     msg = json.loads(data_string)
-                #     flag_dir = os.path.join(FLAGGING_DIRECTORY, str(interface.hash))
-                #     os.makedirs(flag_dir, exist_ok=True)
-                #     output_flag = {'input': interface.input_interface.rebuild_flagged(flag_dir, msg['data']),
-                #               'output': interface.output_interface.rebuild_flagged(flag_dir, processed_output),
-                #               }
-                #     with open(os.path.join(flag_dir, FLAGGING_FILENAME), 'a+') as f:
-                #         f.write(json.dumps(output_flag))
-                #         f.write("\n")
-
                 self.wfile.write(json.dumps(output).encode())
 
                 analytics_thread = threading.Thread(
@@ -197,23 +183,35 @@ def serve_files_in_background(interface, port, directory_to_serve=None, server_n
                 data_string = self.rfile.read(
                     int(self.headers["Content-Length"]))
                 msg = json.loads(data_string)
-                flag_dir = os.path.join(FLAGGING_DIRECTORY,
-                                        str(interface.flag_hash))
-                os.makedirs(flag_dir, exist_ok=True)
+                os.makedirs(interface.flagging_dir, exist_ok=True)
                 output = {'inputs': [interface.input_interfaces[
-                    i].rebuild_flagged(
-                    flag_dir, msg['data']['input_data']) for i
+                    i].rebuild(
+                    interface.flagging_dir, msg['data']['input_data'][i]) for i
                     in range(len(interface.input_interfaces))],
                     'outputs': [interface.output_interfaces[
-                        i].rebuild_flagged(
-                        flag_dir, msg['data']['output_data']) for i
-                    in range(len(interface.output_interfaces))],
-                    'message': msg['data']['message']}
+                        i].rebuild(
+                        interface.flagging_dir, msg['data']['output_data'][i])
+                        for i
+                    in range(len(interface.output_interfaces))]}
 
-                with open(os.path.join(flag_dir, FLAGGING_FILENAME), 'a+') as f:
-                    f.write(json.dumps(output))
-                    f.write("\n")
+                log_fp = "{}/log.csv".format(interface.flagging_dir)
 
+                is_new = not os.path.exists(log_fp)
+
+                with open(log_fp, "a") as csvfile:
+                    headers = ["input_{}".format(i) for i in range(len(
+                        output["inputs"]))] + ["output_{}".format(i) for i in
+                                               range(len(output["outputs"]))]
+                    writer = csv.DictWriter(csvfile, delimiter=',',
+                                            lineterminator='\n',
+                                            fieldnames=headers)
+                    if is_new:
+                        writer.writeheader()
+
+                    writer.writerow(
+                        dict(zip(headers, output["inputs"] +
+                                  output["outputs"]))
+                    )
             else:
                 self.send_error(404, 'Path not found: {}'.format(self.path))
 
@@ -224,22 +222,21 @@ def serve_files_in_background(interface, port, directory_to_serve=None, server_n
             self.base_path = base_path
             BaseHTTPServer.__init__(self, server_address, RequestHandlerClass)
 
+    class QuittableHTTPThread(threading.Thread):
+        def __init__(self, httpd):
+            super().__init__(daemon=False)
+            self.httpd = httpd
+            self.keep_running =True
+
+        def run(self):
+            while self.keep_running:
+                self.httpd.handle_request()
+
     httpd = HTTPServer(directory_to_serve, (server_name, port))
-
-    # Now loop forever
-    def serve_forever():
-        try:
-            while True:
-                sys.stdout.flush()
-                httpd.serve_forever()
-        except (KeyboardInterrupt, OSError):
-                httpd.shutdown()
-                httpd.server_close()
-
-    thread = threading.Thread(target=serve_forever, daemon=False)
+    thread = QuittableHTTPThread(httpd=httpd)
     thread.start()
 
-    return httpd
+    return httpd, thread
 
 
 def start_simple_server(interface, directory_to_serve=None, server_name=None, server_port=None):
@@ -248,8 +245,8 @@ def start_simple_server(interface, directory_to_serve=None, server_name=None, se
     port = get_first_available_port(
         server_port, server_port + TRY_NUM_PORTS
     )
-    httpd = serve_files_in_background(interface, port, directory_to_serve, server_name)
-    return port, httpd
+    httpd, thread = serve_files_in_background(interface, port, directory_to_serve, server_name)
+    return port, httpd, thread
 
 
 def close_server(server):
